/*
 *
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 Maximilian Schnarr <Fyrmir@gmx.de>
 *
 */

#ifndef _LW_LIVEWALLPAPER_PLUGIN_H_
#define _LW_LIVEWALLPAPER_PLUGIN_H_

G_BEGIN_DECLS

#define LW_TYPE_LIVEWALLPAPER_PLUGIN                (lw_livewallpaper_plugin_get_type())
#define LW_LIVEWALLPAPER_PLUGIN(obj)                (G_TYPE_CHECK_INSTANCE_CAST ((obj), LW_TYPE_LIVEWALLPAPER_PLUGIN, LwLivewallpaperPlugin))
#define LW_IS_LIVEWALLPAPER_PLUGIN(obj)             (G_TYPE_CHECK_INSTANCE_TYPE ((obj), LW_TYPE_LIVEWALLPAPER_PLUGIN))
#define LW_LIVEWALLPAPER_PLUGIN_GET_INTERFACE(inst) (G_TYPE_INSTANCE_GET_INTERFACE ((inst), LW_TYPE_LIVEWALLPAPER_PLUGIN, LwLivewallpaperPluginInterface))

typedef struct _LwLivewallpaperPlugin LwLivewallpaperPlugin;
typedef struct _LwLivewallpaperPluginInterface LwLivewallpaperPluginInterface;

/**
 * LwLivewallpaperPluginInterface:
 * @init_plugin: Init function
 * @adjust_viewport: Adjust viewport
 * @prepare_paint: Function to prepare the paint
 * @paint: Paint function
 * @done_paint: Function to clean up after paint
 * @restore_viewport: Restore viewport
 *
 * Interface for plugins providing a live wallpaper.
 */
struct _LwLivewallpaperPluginInterface
{
	/*< private >*/
	GTypeInterface parent_iface;

	/*< public >*/
	void (*init_plugin) (LwLivewallpaperPlugin *self);

	void (*adjust_viewport) (LwLivewallpaperPlugin *self, LwOutput *output);

	void (*prepare_paint) (LwLivewallpaperPlugin *self, int ms_since_last_paint);
	void (*paint) (LwLivewallpaperPlugin *self, LwOutput *output);
	void (*done_paint) (LwLivewallpaperPlugin *self);

	void (*restore_viewport) (LwLivewallpaperPlugin *self);
};

GType lw_livewallpaper_plugin_get_type(void);

void lw_livewallpaper_plugin_init_plugin(LwLivewallpaperPlugin *self);

void lw_livewallpaper_plugin_adjust_viewport(LwLivewallpaperPlugin *self, LwOutput *output);

void lw_livewallpaper_plugin_prepare_paint(LwLivewallpaperPlugin *self, int ms_since_last_paint);
void lw_livewallpaper_plugin_paint(LwLivewallpaperPlugin *self, LwOutput *output);
void lw_livewallpaper_plugin_done_paint(LwLivewallpaperPlugin *self);

void lw_livewallpaper_plugin_restore_viewport(LwLivewallpaperPlugin *self);

G_END_DECLS

#endif /* _LW_LIVEWALLPAPER_PLUGIN_H_ */

