/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>
 */

#include "add_key.h"

static gboolean s_to_rgba  (GValue *,       GVariant *,           gpointer);
static GVariant *rgba_to_s (const GValue *, const GVariantType *, gpointer);

static void     imgchooser (GtkEntry  * /*, gpointer p */);

void
add_key	(GSettings *settings, GtkBox *box, Key *gkey)
{
	GtkWidget *tmpBox, *ctl;
	
	/* Create a box which contains a label with the summary,the tooltip,
	 * the widget... of the key */
	BOX()

	switch (gkey->type)
	{
		case 'b':
			ctl = gtk_switch_new ();
			g_settings_bind (settings, gkey->name, ctl, "active", G_SETTINGS_BIND_DEFAULT);
			gtk_box_pack_end (GTK_BOX(tmpBox), ctl, FALSE, FALSE, 0);
			break;
			
		case 'n': case 'q':		/* int16 uint16 */
		case 'i': case 'u':		/* int   uint   */
		case 'x': case 't':		/* int64 uint32 */
		case 'y': case 'd':		/* byte  double */
			ctl = gtk_spin_button_new_with_range ( gkey->min, gkey->max, gkey->inc);
			
			if (gkey->type == 'd')
				gtk_spin_button_set_digits (GTK_SPIN_BUTTON(ctl), gkey->digits);
				
			g_settings_bind (settings, gkey->name, ctl, "value", G_SETTINGS_BIND_DEFAULT);
			
			gtk_box_pack_end (GTK_BOX(tmpBox), ctl, FALSE, FALSE, 0);
			break;

		case 'c':	/* Color */
			ctl = gtk_color_button_new ();
#if GTK_CHECK_VERSION(3,4,0)
			gtk_color_chooser_set_use_alpha (GTK_COLOR_CHOOSER(ctl), TRUE);
#else
			gtk_color_button_set_use_alpha (GTK_COLOR_BUTTON(ctl), TRUE);
#endif
			g_settings_bind_with_mapping( settings, gkey->name, ctl,
				"rgba", G_SETTINGS_BIND_DEFAULT, s_to_rgba, rgba_to_s, NULL, NULL);

			gtk_box_pack_end (GTK_BOX(tmpBox), ctl, FALSE, FALSE, 0);
			break;
			
		case 'p':	/* Picture */
			ctl = gtk_entry_new ();
			
			g_object_set (ctl, "editable", FALSE, NULL);
			/* Start an image selector onclick */
			g_signal_connect ( ctl, "button-press-event", (GCallback)imgchooser, NULL);
			g_settings_bind (settings, gkey->name, ctl, "text", G_SETTINGS_BIND_DEFAULT);

			gtk_box_pack_end (GTK_BOX(tmpBox), ctl, TRUE, TRUE, 0);
			break;
		
		case 'f':	/* Font */
			ctl = gtk_font_button_new ();
			g_settings_bind (settings, gkey->name, ctl, "font-name", G_SETTINGS_BIND_DEFAULT);
			gtk_box_pack_end (GTK_BOX(tmpBox), ctl, TRUE, TRUE, 0);
			break;
			
		case 's':	/* String */
			ctl = gtk_entry_new ();
			g_settings_bind (settings, gkey->name, ctl, "text", G_SETTINGS_BIND_DEFAULT);
			gtk_box_pack_end (GTK_BOX(tmpBox), ctl, TRUE, TRUE, 0);
			break;
		
		default:
			g_warning ("livewallpaper-config : Unrecognized data-type %c for key %s", gkey->type, gkey->name);
			break;
	}
}

void
reset_key (GtkWidget *w, GSettings *s)
{
	/* Reset the key joined to the GtkButton w in GSettings */
	g_settings_reset (s, g_object_get_data(G_OBJECT(w), "key"));
}

void
cb_bind_get_value (GSettings *s, gchar *key, GtkWidget *w)
{
	/* Get the identifier of the activated choice of the key in GSettings
	 * and set the active choice to this identifier in the GtkComboBoxText */
	gtk_combo_box_set_active (GTK_COMBO_BOX(w),
							  g_settings_get_enum (s, key));
}

void
cb_bind_set_value (GtkWidget *w, GSettings *s)
{
	/* Set the current value of the GtkComboBoxText to GSettings
	 * by the identifier of the value */
	g_settings_set_enum (s, g_object_get_data(G_OBJECT(w), "key"),
						 gtk_combo_box_get_active (GTK_COMBO_BOX(w)));
}

/* Inspirated by gnome-terminal (profile-editor.c ) */
static gboolean
s_to_rgba (GValue *value, GVariant *variant, gpointer p)
{
	const gchar *s;
	GdkRGBA color;
	(void) p;

	g_variant_get (variant, "&s", &s);
	gdk_rgba_parse (&color, s);

	g_value_set_boxed (value, &color);
	return TRUE;
}

/* Inspirated by gnome-terminal (profile-editor.c ) */
static GVariant *
rgba_to_s (const GValue *v, const GVariantType *t, gpointer p)
{
	(void) t; (void) p;
	return g_variant_new_string (gdk_rgba_to_string (g_value_get_boxed (v)));
}

static void
imgchooser (GtkEntry *w /*, gpointer p */)
{
	static gboolean running = FALSE;

	/* If a file selector is already running, we won't open another one */
	if (!running)
	{
		/* Create a GtkFileChooserDialog with two buttons Open and Cancel */
		GtkWidget *dialog = gtk_file_chooser_dialog_new (NULL, NULL, GTK_FILE_CHOOSER_ACTION_OPEN,
													     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
													     GTK_STOCK_OPEN  , GTK_RESPONSE_ACCEPT, NULL);
													     
		/* Create a GtkFileFilter to allow only image files
		 * and set this filter to the dialog file chooser*/
		GtkFileFilter *filter = gtk_file_filter_new ();
		gtk_file_filter_add_pixbuf_formats (filter);
		gtk_file_chooser_set_filter ( GTK_FILE_CHOOSER (dialog), filter);

		/* Avoid to run another file chooser while this one is running */
		running = TRUE;
		
		/* Start the file chooser and wait for a response.
		 * If the response is GTK_RESPONSE_ACCEPT, set the file name to the parent widget */
		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
			gtk_entry_set_text (w, gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog)));

		/* Destroy the file chooser and remove the function locker */
		gtk_widget_destroy (dialog);
		running = FALSE;
	}
}
