/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>
 */

#include <ctype.h>
#include "add_tab.h"

#define PLUGIN_ICON g_strdup_printf(DATADIR "/share/livewallpaper/plugins/%s/icon.svg", plugin)

static void  show_plugin_copyright (GtkWidget *, gchar *);

static void  active_plugin_get (GtkToggleButton *, GSettings *);
static void _active_plugin_get (GSettings *, gchar *, GSList *);

static void   active_plugin_set (GtkToggleButton *, GSettings *);
static void  _active_plugin_set (GtkWidget *, GSettings *);

#define KEY_EXISTS(key)					  g_key_file_has_key   		        (gkf, "Plugin", key, NULL)
#define KEY_STRVAL(key)				      g_key_file_get_locale_string      (gkf, "Plugin", key, NULL, NULL)
#define KEY_STRARRAY(key) (const gchar **)g_key_file_get_locale_string_list (gkf, "Plugin", key, NULL, NULL, NULL)
static void
show_plugin_copyright (GtkWidget *w, gchar *plugin)
{
	GtkAboutDialog *dialog = GTK_ABOUT_DIALOG (gtk_about_dialog_new ());
	gtk_about_dialog_set_logo_icon_name (dialog, "livewallpaper");
	
	if (strcmp (plugin, "livewallpaper"))
	{
		/* First, we initialize the Key-Value File Parser */
		GKeyFile *gkf = g_key_file_new ();
		GError *err = NULL;
		GdkPixbuf *img = gdk_pixbuf_new_from_file (PLUGIN_ICON, &err);
		
		if (!g_key_file_load_from_file (gkf, 
				g_strdup_printf ("%s/.local/lib/livewallpaper/plugins/%s/%s.plugin", g_get_home_dir(), plugin, plugin),
				G_KEY_FILE_NONE, NULL))
		{
			if (! g_key_file_load_from_file (gkf, 
					g_strdup_printf ("/usr/local/lib/livewallpaper/plugins/%s/%s.plugin", plugin, plugin),
					G_KEY_FILE_NONE, NULL))
			{
				if (! g_key_file_load_from_file (gkf, 
						g_strdup_printf ("/usr/lib/livewallpaper/plugins/%s/%s.plugin", plugin, plugin),
						G_KEY_FILE_NONE, NULL))
				{
					return;
				}
			}
		}

		if (err == NULL) gtk_about_dialog_set_logo (dialog, img);
		else             g_error_free (err);

		/* Then, we add to the dialog the data we've found */
		gtk_about_dialog_set_program_name (dialog, KEY_STRVAL("Name"));
		gtk_about_dialog_set_comments     (dialog, KEY_STRVAL("Description"));
			
		gtk_about_dialog_set_authors   (dialog, KEY_STRARRAY ("Authors"));
		gtk_about_dialog_set_copyright (dialog, KEY_STRVAL("Copyright"));
			
		if (KEY_EXISTS("Website"))
		{
			gtk_about_dialog_set_website       (dialog, KEY_STRVAL("Website"));
			gtk_about_dialog_set_website_label (dialog, _("Website"));
		}
	
		g_key_file_free (gkf);
		(void) w;
	}
	else
	{
		const gchar *authors[] = {"Maximilian Schnarr <Fyrmir@gmx.de>", "Aurélien RIVIERE <aurelien.riv@gmail.com>", NULL};
		
		gtk_about_dialog_set_program_name (dialog, _("Livewallpaper"));
		gtk_about_dialog_set_comments (dialog, _("Replace your boring default wallpaper with a completely animated OpenGL wallpaper."));
		
		gtk_about_dialog_set_authors (dialog, authors);
		gtk_about_dialog_set_copyright (dialog, "Copyright © 2012 Maximilian Schnarr");
		
		gtk_about_dialog_set_website (dialog, "http://launchpad.net/livewallpaper");
		gtk_about_dialog_set_website_label (dialog, _("Website"));
	}
	
    /* We run the dialog */
    switch (gtk_dialog_run (GTK_DIALOG (dialog)))
    {
		case GTK_RESPONSE_CANCEL:
		case GTK_RESPONSE_DELETE_EVENT: 
			gtk_widget_destroy(GTK_WIDGET(dialog));
			break;
	};
}

static void
active_plugin_get (GtkToggleButton *w, GSettings *s)
{
	/* Check if the widget's embeded string is equals to
	 * the name of the active-plugin property, and mark
	 * the widget as active if it is so. */
	if (! strcmp (g_settings_get_string (s, "active-plugin"), 
				  g_object_get_data (G_OBJECT(w), "plugin")))
		gtk_toggle_button_set_active (w, TRUE);
}
static void
_active_plugin_get (GSettings *s, gchar *key, GSList *l)
{
	/* Call active_plugin_get foreach element of the GSList */
	g_slist_foreach (l, (GFunc)active_plugin_get, s);
	(void) key;
}

static void
active_plugin_set (GtkToggleButton *w, GSettings *s)
{
	/* Check if the widget is active, and if it is so,
	 * set the active-plugin property to its new value */
	if ( gtk_toggle_button_get_active (w))
		g_settings_set_string (s, "active-plugin",
							   g_object_get_data (G_OBJECT(w), "plugin"));
}
static void
_active_plugin_set (GtkWidget *w, GSettings *s)
{
	/* Get the GSList associated to the widget activated and
	 * call active_plugin_set foreach element of this GSList */
	GSList *l = gtk_radio_button_get_group (GTK_RADIO_BUTTON (w));
	g_slist_foreach (l, (GFunc)active_plugin_set, s);
}

void
add_tab (GtkNotebook *notebook, GtkWidget *nb, gchar *plugin)
{
	static GSList *group = NULL;
	GSettings *settings  = g_settings_new (GSETTINGS_PREFIX);
	GtkWidget *box       = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5),
			  *tmpBox    = gtk_box_new (GTK_ORIENTATION_VERTICAL,   5),
			  *ctl;
				
	GError *err = NULL;
	GdkPixbuf *icon = gdk_pixbuf_new_from_file (PLUGIN_ICON, &err);
	if (err == NULL)
		ctl = gtk_image_new_from_pixbuf (icon);
	else
	{
		ctl = gtk_image_new_from_icon_name ("livewallpaper-config", GTK_ICON_SIZE_DIALOG);
		g_error_free (err);
	}

	/* Tab Image */
	gtk_box_pack_start(GTK_BOX(box), ctl, FALSE, FALSE, 0);

	/* Tab Label */
	gtk_box_pack_start( GTK_BOX(box),
						gtk_label_new ( g_strdup_printf ("%c%s", toupper(plugin[0]), plugin+1) ),
						TRUE, TRUE, 0);

	if (!strcmp(plugin, "livewallpaper"))
	{	
		ctl = gtk_switch_new ();
		g_settings_bind (settings, "active", ctl, "active", G_SETTINGS_BIND_DEFAULT);
		/* Insert this page in first */
		gtk_notebook_insert_page (notebook, nb, box, 0);
	}
	else
	{
		/* Create a GtkRadioButton and join it to the GSList group */
		ctl = gtk_radio_button_new (group);
		group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (ctl));
		
		gtk_widget_set_tooltip_text (ctl, _("Define which plugin is currently enabled."));
		
		/* Add a key "plugin" to the widget to get the string to set
		 * to the active-plugin property when the widget is activated,
		 * connect the signal "toggled" to be notified or any change
		 * connect the "changed" signal for key active-plugin in
		 * for settings to be notified of any change of this key */
		g_object_set_data (G_OBJECT (ctl), "plugin",  g_strdup (plugin));
		g_signal_connect ( G_OBJECT (ctl), "toggled", (GCallback)_active_plugin_set, settings);
		g_signal_connect (settings, "changed::active-plugin", (GCallback)_active_plugin_get, group);
		_active_plugin_get (settings, NULL, group);
		
		gtk_notebook_append_page (notebook, nb, box);
	}
	
	gtk_box_pack_end (GTK_BOX(tmpBox), ctl, TRUE, FALSE, 5);
	
	/* Add a button to display plugin's informations in a toplevel window */
	ctl = gtk_button_new ();
	gtk_button_set_image (GTK_BUTTON(ctl),
						  gtk_image_new_from_stock (GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU));
	g_signal_connect ( G_OBJECT (ctl), "clicked", (GCallback)show_plugin_copyright, plugin);
	gtk_box_pack_end(GTK_BOX(box), ctl, FALSE, FALSE, 0);
	
	gtk_box_pack_end (GTK_BOX(box), tmpBox, FALSE, FALSE, 10);

	gtk_widget_show_all(box);
}
#undef ICON
