/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>
 */

#include "loader.h"

static guint nb_decimals  (gdouble x);
static void  load_enum    (GSettings *, GtkBox *, Key *, FILE *);
static void  parse_schema (GtkNotebook *, const gchar *);

static guint
nb_decimals  (gdouble x)
{
	gfloat f = x * 10e6;
	guint n = 7, /* = number of decimals */
		  b = 10;

	while ( (gint)f % b == 0 && n > 0)
	{
		b = b * 10;
		n--;
	}

	return n;
}

static void
load_enum (GSettings *settings, GtkBox *box, Key *gkey, FILE *file)
{
	GtkWidget *tmpBox, *ctl;
	
	gchar str[1024];
	
	/* If pass == FALSE, look for the enum and ignore found values
	 * Else, enum found, read its values */
	gboolean pass = FALSE;

	/* Remember the current position in the file
	 * to be able to restore the position later,
	 * and go to the beginning of the file */
	guint pos = ftell (file);
	rewind (file);

	/* Create a box which contains a label with the summary,the tooltip,
	 * the widget... of the key, and create a GtkComboBoxText */
	BOX()
	ctl = gtk_combo_box_text_new ();

	/* Read the entire file line by line */
	while (fgets(str, 1024, file))
	{
		if (strstr(str, "<enum"))
		{
			gchar enum_id[64];
			sscanf(strpbrk(str,"<"), "<enum id=%*c%63[^\"\">\"]%*c", enum_id);
			/* If enum_id is equals to the expected enum (gkey->enum_id), pass = TRUE
			 * Else pass = FALSE */
			pass = !strcmp(enum_id, gkey->enum_id);
		}
		else if (pass && strstr(str, "<value"))
		{
			gint value;
			gchar nick[128];
			
			/* Try to read the arguments in any orders */
			if ( sscanf(strpbrk(str,"<"), "<value nick=%*c%127[^\"\">\"]%*c value=%*c%i%*c", nick, &value)  != 2)
				 sscanf(strpbrk(str,"<"), "<value value=%*c%i%*c nick=%*c%127[^\"\">\"]%*c", &value, nick);
				 
			/* Add the found values to the GtkComboBoxText with value as identifier and nick as displayed string (translated) */
			gtk_combo_box_text_insert_text (GTK_COMBO_BOX_TEXT(ctl), value, _(nick));
		}
		else if (pass && strstr(str,"</enum>"))
			break;
	}
	
	/* Restore the original position in the file */
	fseek (file, pos, SEEK_SET);
	
	/* Join the key name to the GtkComboBoxText */
	g_object_set_data(G_OBJECT(ctl), "key", g_strdup(gkey->name));
	
	/* Update the widget */
	cb_bind_get_value (settings, gkey->name, ctl);
	
	/* Bind the "changed "signal to the widget and the "changed::name" to gsettings */
	g_signal_connect (ctl, "changed", (GCallback)cb_bind_set_value, settings);
	g_signal_connect (settings,
					  g_strdup_printf("changed::%s", gkey->name),
					  (GCallback)cb_bind_get_value, ctl);

	/* Place the widget in the GUI */
	gtk_box_pack_end (GTK_BOX(tmpBox), ctl, FALSE, FALSE, 0);
}

static void
parse_schema (GtkNotebook *notebook, const gchar *gschema)
{
	FILE *file = file = fopen(g_strdup_printf("%s/.local/share/glib-2.0/schemas/%s.gschema.xml", g_get_home_dir(), gschema), "r");
	if (! file)	
		file = fopen(g_strdup_printf("/usr/local/share/glib-2.0/schemas/%s.gschema.xml", gschema), "r");
	if (! file)
		file = fopen(g_strdup_printf("/usr/share/glib-2.0/schemas/%s.gschema.xml", gschema), "r");
		
	if (! file)
		fprintf(stderr, "livewallpapers-config : Unable to locate schema %s", gschema);
	else
	{
		GSettings *settings = g_settings_new (gschema);
		Key gkey;		 /* Where data will be stored */
		gchar str[1024];
		
		/* The tab which will countain the keys and be scolled 
		 * Reloaded on each tab of nb */
		GtkWidget *tab = NULL;
		
		/* The per-plugin notebook */
		GtkNotebook *nb  = GTK_NOTEBOOK(gtk_notebook_new());
		gtk_notebook_set_scrollable (nb, TRUE);

		while (fgets(str, 1024, file))
		{
			if (strstr(str, "<lwp:tab"))
			{
				/* Create a scolled area for the keys and add it in a new tab */
				
				GtkWidget *scroll = gtk_scrolled_window_new (NULL, NULL);

				gchar tab_name[32];
				sscanf(strpbrk(str,"n"), "name=%*c%31[^\"\">\"]%*c", tab_name);

				tab = gtk_box_new (GTK_ORIENTATION_VERTICAL, 5);
				gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
				gtk_scrolled_window_add_with_viewport ( GTK_SCROLLED_WINDOW (scroll), tab);
				gtk_notebook_append_page (nb, scroll, gtk_label_new(_(tab_name)));
			}

			else if (strstr(str, "<lwp:separator"))
				/* Add an horizontal separator to separate widgets inside a same tab */
				gtk_box_pack_start (GTK_BOX(tab),
								  gtk_separator_new(GTK_ORIENTATION_HORIZONTAL),
								  TRUE, TRUE, 2);
			
			else if (strstr(str,"<key "))
			{
				if (strstr(str,"type="))
				{
					/* Get the type and the name of the new key,
					 * and set the number of digits and the incrementation
					 * to a default value */
					if ( sscanf(strpbrk(str,"t"), "type=%*c%c%*c name=%*c%31[^\"\">\"]%*c", &gkey.type, gkey.name) != 2)
						 sscanf(strpbrk(str,"n"), "name=%*c%31[^\"\">\"]%*c type=%*c%c%*c", gkey.name, &gkey.type);
					gkey.digits = 6;
					gkey.inc    = (gkey.type == 'd') ? 10e-7 : 1;
				}
				else if (strstr(str, "enum="))
				{
					/* Get the name of an enum key and the associated enum,
					 * and set type to 'e' (not reserved by glib types and
					 * permit to know the key needs load_enum instead of add_key) */
					if ( sscanf(strpbrk(str,"n"), "name=%*c%31[^\"\">\"]%*c enum=%*c%127[^\"\">\"]%*c", gkey.name, gkey.enum_id) != 2)
						 sscanf(strpbrk(str,"e"), "enum=%*c%127[^\"\">\"]%*c name=%*c%31[^\"\">\"]%*c", gkey.enum_id, gkey.name);
					gkey.type = 'e';
				}
				
				gkey.skip = FALSE;
			}
			else if (strstr(str,"<lwp:type"))
			{
				gchar buffer[16];
				sscanf(strpbrk(str,">"), ">%15[^\"</\"]", buffer);

				if      (! strcmp(buffer, "image")) gkey.type = 'p';
				else if (! strcmp(buffer, "color")) gkey.type = 'c';
				else if (! strcmp(buffer, "font"))  gkey.type = 'f';
			}
			
			else if (strstr(str,"<lwp:skip"))
				gkey.skip = TRUE;
				
			else if (strstr(str,"<range"))
				sscanf(strpbrk(str,"m"), "min=%*c%f%*c max=%*c%f%*c", &gkey.min, &gkey.max);

			else if (strstr(str,"<summary>"))
				sscanf(strpbrk(str,">"), ">%255[^\"</\"]", gkey.summary);

			else if (strstr(str,"<description>"))
				sscanf(strpbrk(str,">"), ">%511[^\"</\"]", gkey.description);

			else if (strstr(str, "<lwp:increment"))
				/* The increment step of the buttons + and - when we do a button1 click 
				 * button2 click = 10*button1 click */
				sscanf(strpbrk(str,">"), ">%f", &gkey.inc);

			else if (strstr(str, "<lwp:digits"))
				/* The number of decimals displayed */
				sscanf(strpbrk(str,">"), ">%u", &gkey.digits);

			else if (strstr(str,"</key>") && !gkey.skip)
			{
				if (gkey.type == 'e')
					load_enum (settings, GTK_BOX(tab), &gkey, file);
				else
				{
					if (gkey.type == 'd')
						if ( nb_decimals (gkey.inc) > gkey.digits)
							gkey.inc = pow (10.0, - (gdouble)gkey.digits);

					add_key (settings, GTK_BOX(tab), &gkey);
				}
			}
		}
		fclose (file);
		
		/* Add the per-plugin notebook to the main notebook and
		 * add the GtkRadioButton which permit to select the active-plugin */
		add_tab (notebook, GTK_WIDGET(nb),
			g_strdup ((strcmp(gschema, GSETTINGS_PREFIX)) ? gschema + GSETTINGS_PREFIX_LEN : "livewallpaper"));
	}
}

void
load_plugins (GtkNotebook *w)
{
	guint i;
	const gchar **schemas = (const gchar **) g_settings_list_schemas();

	/* For each installed schema,
	 * if the schema has the prefix of a livewallpaper's plugin,
	 * load it */
	for (i=0; schemas[i]; i++)
		if (g_str_has_prefix (schemas[i], GSETTINGS_PREFIX))
			parse_schema (w, schemas[i]);
}
