/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>
 */

#include "main.h"

static void create_gui (GtkApplication *);

static void
create_gui (GtkApplication *app)
{
    GList *list = gtk_application_get_windows (app);

    if (list)	/* If an instance of the app is running, we put its window to the foreground */
	gtk_window_present (GTK_WINDOW (list->data));
    else 	/* Else we create the window */
    {
		GtkNotebook *notebook = GTK_NOTEBOOK(gtk_notebook_new());
		
#if GTK_CHECK_VERSION(3,4,0)
		GtkWindow *window = GTK_WINDOW(gtk_application_window_new (app));
#else
		GtkWindow *window = GTK_WINDOW(gtk_window_new (GTK_WINDOW_TOPLEVEL));
		gtk_window_set_application (window, app);
#endif
		gtk_window_set_title        (window, _("LiveWallpaper Preferences"));
		gtk_window_set_icon_name    (window, "livewallpaper-config");
		gtk_window_set_default_size (window, 800, 200);
		
		gtk_notebook_set_tab_pos    (notebook, GTK_POS_LEFT);
		gtk_notebook_set_scrollable (notebook, TRUE);
		
		load_plugins (notebook);

		gtk_container_add   (GTK_CONTAINER(window), GTK_WIDGET(notebook));
		gtk_widget_show_all (GTK_WIDGET(window));
    }
}

gint
main (gint argc, gchar **argv)
{
	/* Create a GtkApplication with a unique ID to permit running in single instance mode */
    GtkApplication *app = gtk_application_new (GSETTINGS_PREFIX"livewallpaper-config", G_APPLICATION_FLAGS_NONE);

	/* Preparing Gettext */
    setlocale      (LC_ALL, "");
    textdomain     (TEXT_DOMAIN);
    bindtextdomain (TEXT_DOMAIN, DATADIR"/share/locale");

	/* Start the application as a GApplication */
    g_signal_connect (app, "activate", (GCallback)create_gui, NULL);
    return g_application_run ((GApplication *)app, argc, argv);
}
