/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>
 */

#include "main.h"
#include "plugins.h"

#define SEPARATOR()	gtk_menu_shell_append(menu,gtk_separator_menu_item_new())

static void gui (void);	/* (GtkWidget *, gpointer); */

static void
gui (void) /* (GtkWidget *w, gpointer p); */
{
#if GLIB_CHECK_VERSION(2,32,0)
    g_thread_new (NULL, (GThreadFunc)system, (gpointer)"livewallpaper-config");
#else
    g_thread_create (   (GThreadFunc)system, (gpointer)"livewallpaper-config", FALSE, NULL);
#endif
}

gint
main (gint argc, gchar **argv)
{
    AppIndicator *indicator;
    GtkMenuShell *menu;
    GtkWidget *ctl;

    /* Preparing Gettext */
    setlocale(LC_ALL, "");
    textdomain(TEXT_DOMAIN);
    bindtextdomain(TEXT_DOMAIN, DATADIR"/share/locale");

    gtk_init (&argc, &argv);

    indicator = app_indicator_new ( "LiveWallpaper", "livewallpaper-indicator",
									APP_INDICATOR_CATEGORY_APPLICATION_STATUS);
    app_indicator_set_status (indicator, APP_INDICATOR_STATUS_ACTIVE);

    menu = GTK_MENU_SHELL (gtk_menu_new ());

    /* Live Wallpaper checkbutton */
    ctl = gtk_check_menu_item_new_with_label (_("LiveWallpaper"));
    g_settings_bind (g_settings_new (GSETTINGS_PREFIX), "active", ctl,
		     "active", G_SETTINGS_BIND_DEFAULT);
    gtk_menu_shell_append (menu, ctl);

    SEPARATOR();
    load_plugins(menu);
    SEPARATOR();

    /* Add a link to the configuration GUI */
    ctl = gtk_menu_item_new_with_label (_("LiveWallpaper Settings..."));
        g_signal_connect (G_OBJECT(ctl), "activate", G_CALLBACK (gui), NULL);
        gtk_menu_shell_append (menu, ctl);

    app_indicator_set_menu (indicator, GTK_MENU(menu));
    gtk_widget_show_all(GTK_WIDGET(menu));
    gtk_main ();

    return EXIT_SUCCESS;
}
