/*
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 RIVIÈRE Aurélien <aurelien.riv@gmail.com>
 */

#include "plugins.h"

static void  active_plugin_get (GtkWidget *, GSettings *);
static void _active_plugin_get (GSettings *, gchar *, GSList *);

static void   active_plugin_set (GtkWidget *, GSettings *);
static void  _active_plugin_set (GtkWidget *, GSettings *);

static void
active_plugin_get (GtkWidget *w, GSettings *s)
{
	/* Check if the widget's embeded string is equals to
	 * the name of the active-plugin property, and mark
	 * the widget as active if it is so. */
	if (! strcmp (g_settings_get_string (s, "active-plugin"), 
				  g_object_get_data (G_OBJECT(w), "plugin")))
		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (w), TRUE);
}
static void
_active_plugin_get (GSettings *s, gchar *key, GSList *l)
{
	/* Call active_plugin_get foreach element of the GSList */
	g_slist_foreach (l, (GFunc)active_plugin_get, s);
	(void) key;
}

static void
active_plugin_set (GtkWidget *w, GSettings *s)
{
	/* Check if the widget is active, and if it is so,
	 * set the active-plugin property to its new value */
	if ( gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (w)))
		g_settings_set_string (s, "active-plugin",
							   g_object_get_data (G_OBJECT(w), "plugin"));
}
static void
_active_plugin_set (GtkWidget *w, GSettings *s)
{
	/* Get the GSList associated to the widget activated and
	 * call active_plugin_set foreach element of this GSList */
	GSList *l = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (w));
	g_slist_foreach (l, (GFunc)active_plugin_set, s);
}

void
load_plugins (GtkMenuShell *menu)
{
    const gchar **schemas = (const gchar **) g_settings_list_schemas();
    GSettings *settings = g_settings_new(GSETTINGS_PREFIX);
    GSList *group = NULL;
	gint i;
	
	/* For each GSettings schemas installed on the system,
	 * if the schema has the prefix of a livewallpaper's plugin,
	 * add a radiobutton */
    for (i=0; schemas[i]; i++)
		if ( g_str_has_prefix (schemas[i], GSETTINGS_PREFIX"."))
		{
			/* label : first letter of the name in uppercase,
			 * 		   followed by the rest of the name without changes */
			GtkWidget *ctl = gtk_radio_menu_item_new_with_label (group,
								_( g_strdup_printf("%c%s",
									toupper(schemas[i][GSETTINGS_PREFIX_LEN]),
									schemas[i]+GSETTINGS_PREFIX_LEN+1)));
			group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (ctl));
									
			/* Add a key "plugin" to the widget to get the string to set
			 * to the active-plugin property when the widget is activated,
			 * and connect the signal "toggled" to be notified or any changes */
			g_object_set_data (G_OBJECT (ctl), "plugin",  g_strdup (schemas[i]+GSETTINGS_PREFIX_LEN));
			g_signal_connect ( G_OBJECT (ctl), "toggled", (GCallback)_active_plugin_set, settings);

			/* Add the widget to the indicator's menu */
			gtk_menu_shell_append (menu, ctl);
		}
	/* connect the "changed" signal for key active-plugin in
	 * for settings to be notified of any change of this key,
	 * and initialize the current value of the GtkRadioMenuItem group */
	g_signal_connect (settings, "changed::active-plugin", (GCallback)_active_plugin_get, group);
	_active_plugin_get (settings, NULL, group);
}
