/*
 *
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 Maximilian Schnarr <Fyrmir@gmx.de>
 *
 */

#ifndef _NEXUS_PARTICLE_H_
#define _NEXUS_PARTICLE_H_

G_BEGIN_DECLS

typedef enum
{
	NexusGlowTypeRadial = 0,
	NexusGlowTypeSquare,
	NexusGlowTypeSpiral,
	NexusGlowTypeConcentricCircles
} NexusGlowType;

#define NEXUS_TYPE_PARTICLE_SYSTEM            (nexus_particle_system_get_type())
#define NEXUS_PARTICLE_SYSTEM(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), NEXUS_TYPE_PARTICLE_SYSTEM, NexusParticleSystem))
#define NEXUS_IS_PARTICLE_SYSTEM(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NEXUS_TYPE_PARTICLE_SYSTEM))
#define NEXUS_PARTICLE_SYSTEM_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), NEXUS_TYPE_PARTICLE_SYSTEM, NexusParticleSystemClass))
#define NEXUS_IS_PARTICLE_SYSTEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), NEXUS_TYPE_PARTICLE_SYSTEM))
#define NEXUS_PARTICLE_SYSTEM_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), NEXUS_TYPE_PARTICLE_SYSTEM, NexusParticleSystemClass))

typedef struct _NexusParticleSystem NexusParticleSystem;
typedef struct _NexusParticleSystemClass NexusParticleSystemClass;

typedef struct _NexusParticleSystemPrivate NexusParticleSystemPrivate;

struct _NexusParticleSystem
{
	GObject parent_instance;

	/*< private >*/
	NexusParticleSystemPrivate *priv;
};

struct _NexusParticleSystemClass
{
	GObjectClass parent_class;
};

GType nexus_particle_system_get_type(void);

/**
 * nexus_particle_system_set_pulse_count:
 * @self: self
 */
void nexus_particle_system_set_pulse_count(NexusParticleSystem *self, guint count);

/**
 * nexus_particle_system_get_pulse_count:
 * @self: self
 */
guint nexus_particle_system_get_pulse_count(NexusParticleSystem *self);

/**
 * nexus_particle_system_set_pulse_size:
 * @self: self
 */
void nexus_particle_system_set_pulse_size(NexusParticleSystem *self, guint size);

/**
 * nexus_particle_system_get_pulse_size:
 * @self: self
 */
guint nexus_particle_system_get_pulse_size(NexusParticleSystem *self);

/**
 * nexus_particle_system_set_max_length:
 * @self: self
 * @length:
 */
void nexus_particle_system_set_max_length(NexusParticleSystem *self, double length);

/**
 * nexus_particle_system_get_max_length:
 * @self: self
 */
double nexus_particle_system_get_max_length(NexusParticleSystem *self);

/**
 * nexus_particle_system_set_min_length:
 * @self: self
 * @length:
 */
void nexus_particle_system_set_min_length(NexusParticleSystem *self, double length);

/**
 * nexus_particle_system_get_min_length:
 * @self: self
 */
double nexus_particle_system_get_min_length(NexusParticleSystem *self);

/**
 * nexus_particle_system_set_max_delay:
 * @self: self
 * @delay:
 */
void nexus_particle_system_set_max_delay(NexusParticleSystem *self, guint delay);

/**
 * nexus_particle_system_get_max_delay:
 * @self: self
 */
guint nexus_particle_system_get_max_delay(NexusParticleSystem *self);

/**
 * nexus_particle_system_set_texture_dir:
 * @self: self
 * @texturedir:
 */
void nexus_particle_system_set_texture_dir(NexusParticleSystem *self, gchar *texturedir);

/**
 * nexus_particle_system_get_texture_dir:
 * @self: self
 */
gchar *nexus_particle_system_get_texture_dir(NexusParticleSystem *self);

/**
 * nexus_particle_system_set_glow_type:
 * @self: self
 * @type:
 */
void nexus_particle_system_set_glow_type(NexusParticleSystem *self, NexusGlowType type);

/**
 * nexus_particle_system_get_glow_type:
 * @self: self
 */
NexusGlowType nexus_particle_system_get_glow_type(NexusParticleSystem *self);

/**
 * nexus_particle_system_set_color:
 * @self: self
 * @index:
 * @color:
 */
void nexus_particle_system_set_color(NexusParticleSystem *self, int index, GdkRGBA *color);

/**
 * nexus_particle_system_get_color:
 * @self: self
 * @index:
 * @color:
 */
void nexus_particle_system_get_color(NexusParticleSystem *self, int index, GdkRGBA *color);

/**
 * nexus_particle_system_update:
 * @self: self
 */
void nexus_particle_system_update(NexusParticleSystem *self, int ms_since_last_paint);

/**
 * nexus_particle_system_draw:
 * @self: self
 * @size:
 */
void nexus_particle_system_draw(NexusParticleSystem *self, int size);

G_END_DECLS

#endif /* _NEXUS_PARTICLE_H_ */

