/*
 *
 * Livewallpaper
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2012 Maximilian Schnarr <Fyrmir@gmx.de>
 *
 */

#include "config.h"

#include <glib.h>
#include <gdk/gdk.h>
#include <girepository.h>

#include "application.h"


gint
main(gint argc, gchar *argv[])
{
	GApplication *g_app;
	LwApplication *app;
	gchar *local_schemadir, *typelibdir;
	GError *error = NULL;
	int retval;

	/* Initialize external libraries */
	g_type_init();
	gdk_init(&argc, &argv);

	/* Check for application uniqueness */
	g_app = g_application_new("net.launchpad.Livewallpaper", G_APPLICATION_FLAGS_NONE);
	if(!g_application_register(g_app, NULL, &error))
	{
		g_critical("Could not register application: %s", error->message);
		g_error_free(error);
		return 1;
	}

	if(g_application_get_is_remote(g_app))
	{
		g_critical("Could not start livewallpaper application: "
			"Another livewallpaper instance already appears to be running.");
		return 1;
	}

	/* Enable local settings */
	local_schemadir = g_build_filename(g_get_home_dir(), ".local", "share", "glib-2.0", "schemas", NULL);
	g_setenv("GSETTINGS_SCHEMA_DIR", local_schemadir, TRUE);
	g_free(local_schemadir);

	/* Load required repositories */
	if(!g_irepository_require(g_irepository_get_default(), "Peas", "1.0", 0, &error))
	{
		g_critical("Could not load Peas repository: %s", error->message);
		g_error_free(error);
		return 1;
	}

	typelibdir = g_build_filename(LIBDIR, "livewallpaper", "girepository-1.0", NULL);
	if(!g_irepository_require_private(g_irepository_get_default(), typelibdir, "LW", "1.0", 0, &error))
	{
		g_critical("Could not load LW repository: %s", error->message);
		g_error_free(error);
		return 1;
	}
	g_free(typelibdir);

	/* Run livewallpaper application */
	app = g_object_new(LW_TYPE_APPLICATION, NULL);

	retval = lw_application_run(app, argc, argv);

	/* Exit application */
	g_object_unref(app);
	g_object_unref(g_app);

	return retval;
}

